PRO syntool, OBSfile=OBSfile, OBSdir=OBSdir, inputdirFUV=inputdirFUV, inputdirNUV=inputdirNUV, $
    inputfileFUV=inputfileFUV, inputfileNUV=inputfileNUV, dtimeSIM=dtimeSIM, dtimend=dtimend, $
    SJIframe=SJIframe, rasterSJI=rasterSJI, maxWINsize=maxWINsize, $
    savefitsfiles=savefitsfiles, saveplot=saveplot, savelogcsv=savelogcsv, savelogtxt=savelogtxt, $
    copyxml=copyxml, makemovie=makemovie, playmovie=playmovie, quicktimeloc=quicktimeloc, $
    threshold_Overview_SJI=threshold_Overview_SJI, threshold_SJI=threshold_SJI, threshold_Spec=threshold_Spec, $
    debug=debug, debtime=debtime
  ;+
  ; NAME:
  ;       SYNTOOL
  ;
  ; PURPOSE:
  ;       simulates an IRIS observation series
  ;       logics included:
  ;         -reads xml-input files, and copies these files into target-folder if desired
  ;         -reads data input files from simulation; SJI: FUV1, FUV2, NUV1, (NUV2 missing); Spectras: FUV1, FUV2, NUV
  ;         -handles the PZT-logic of the instrument
  ;         -interpolation of input data in all dimensions, except wavelength (i.e. time, x, y)
  ;         -binning of data
  ;         -input data is multiplied by exposure time (assumed input: datacount/s, output: datacount)
  ;         -fits-files are created, data is zeroed outside of CRS-windows
  ;         -FUV, NUV and SJI get each a separate file
  ;         -keywords are included in header
  ;
  ; CALLING SEQUENCE:
  ;       syntool
  ;
  ; INPUT PARAMETER:
  ;       Are so far given within this procedure, see further below
  ;
  ; KEYWORDS:
  ;       none
  ;
  ; OUTPUTS:
  ;       none
  ;
  ; COMMON BLOCKS:
  ;       none
  ;
  ; MODIFICATION HISTORY:
  ;       11-Jun-2012  M. Wiesmann, UIO
  ;       works best with xml-files created by irisplantool_20121017.jar by Ankur Somani (LMSAL)
  ;       works also with later versions, it was tested also with build irisplantool_20121030
  ;-
    
    
  ; INPUT
  if ~keyword_set(OBSfile) then begin
    ;OBSfile = 'OBS-T-00901_Q.xml'   ; The OBS filename (e.g. 'OBS-T-00004.xml')
    if keyword_set(OBSdir) then path=OBSdir else path=''
    file=dialog_pickfile(/read, filter=['OBS*.xml'], path=path)
    if file eq '' then return
    OBSfile = file_basename(file)
    OBSdir = file_dirname(file, /mark_directory)
  endif else begin
    if ~keyword_set(OBSdir) then begin
      ;OBSdir = 'XML_bart_121025/OBS/'       ; OBS-file directory (e.g. 'XMLfiles/')
      OBSdir = file_dirname(OBSfile, /mark_directory)
      OBSfile = file_basename(OBSfile)
    endif
  endelse
  if ~N_ELEMENTS(inputdirFUV) then $
    inputdirFUV = 'Data_cb24bih/' ; directory of input FUV files (e.g. 'Data_cb24bih/')
  if ~N_ELEMENTS(inputdirNUV) then $
    inputdirNUV = 'Data_cb24bih/' ; directory of input NUV files (e.g. 'Data_cb24bih/')
  if ~N_ELEMENTS(inputfileFUV) then $
    inputfileFUV = 'iris_synthetic_fuv_*.fits' ; format of input FUV files (as used in wildcards or strmatch, where * is placed where the file number is, e.g. 'iris_synthetic_fuv_*.fits')
  if ~N_ELEMENTS(inputfileNUV) then $
    inputfileNUV = 'iris_synthetic_nuv_cb24bih_*.fits' ; format of input FUV files (as used in wildcards or strmatch, where * is placed where the file number is, e.g. 'iris_synthetic_nuv_cb24bih_*.fits')
  ; assumptions about the input files:
  ; - they all have the same filename length (that is, FUV and NUV can have different filename lengths, but all FUV, or NUV respectively, have the same length)
  ; - file numbers are between 100 and 999
  ; - there are no other 3- or more digit numbers in the filename
  ; - dtimeSIM, defined below, is valid for consecutive numbers for all files (time-equidistance)
  if ~N_ELEMENTS(dtimeSIM) then $
    dtimeSIM = 10000L         ; time steps between simulation files (in ms)
  if ~N_ELEMENTS(dtimend) then $
    dtimeSIMend = 30000L $     ; time step between last and first file (in case OBS is longer than the time series from the simulation)
  else dtimeSIMend=dtimend
  if ~N_ELEMENTS(SJIframe) then $
    SJIframe=2d               ; visible data around overview SJI (in arcsec), 'Frame'
  if ~N_ELEMENTS(maxWINsize) then $
    maxWINsize=[1400, 1000]   ; max size of plot window [x, y]
  if ~N_ELEMENTS(rasterSJI) then $
    rasterSJI='fu1'           ; ('fu1', 'fu2', 'nu1', 'nu2') choose what SJI data is shown in the overview SJI graph (which will be updated on every step), NUV2 not available yet
    
  ;some keywords
  if ~N_ELEMENTS(savefitsfiles) then $
    savefitsfiles=1 ;set to 1 if you want to save the fits files
  if ~N_ELEMENTS(saveplot) then $
    saveplot=1      ;set to 1 if you want to save the images, otherwise they will be plotted on screen
  if ~N_ELEMENTS(makemovie) then $
    makemovie=1     ;set to 1 if you want to create the movie out of the pictures (saveplot should be 1), qt-tools required (http://omino.com/sw/qt_tools/), MAC only
  if ~N_ELEMENTS(playmovie) then $
    playmovie=1     ;set to 1 if you want to play the movie immediately after creating, MAC only
  if ~N_ELEMENTS(quicktimeloc) then $
    quicktimeloc='/Applications/QuickTime\ Player.app/Contents/MacOS/QuickTime\'  ;location of your quicktime application, required for playmovie, MAC only
  if ~N_ELEMENTS(savelogcsv) then $
    savelogcsv=1    ;set to 1 if you want to save a logfile in csv-format (excel-readable)
  if ~N_ELEMENTS(savelogtxt) then $
    savelogtxt=1    ;set to 1 if you want to save a logfile in txt-format (human-readable)
  if ~N_ELEMENTS(copyxml) then $
    copyxml=1       ;set to 1 if you want to copy the xml-files to the destination folder
    
  ;for debugging:
  if ~N_ELEMENTS(debug) then begin
    debug=0         ;set to 1 to control directly, how many pictures will be calculated
    nrpics=1        ;set the number of pictures to be calculated in debug-mode (program stops when this number is reached)
  endif else nrpics=debug
  if ~N_ELEMENTS(debtime) then $
    debugtime=0 $    ;this time will be added to the real time in the plot output, when in debug-mode (in ms)
  else debugtime=debtime
  
  ;graph variables
  if ~N_ELEMENTS(threshold_Overview_SJI) then $
    threshold_SJI_Overview=0.0002 $;threshold for display used in histo_opt for overview SJI
  else threshold_SJI_Overview=threshold_Overview_SJI
  if ~N_ELEMENTS(threshold_SJI) then $
    threshold_SJI=0.002          ;threshold for display used in histo_opt for SJI(s)
  if ~N_ELEMENTS(threshold_Spec) then $
    threshold_Spec=0.02         ;threshold for display used in histo_opt for spectral images
    
  ;more graph variables
  pcharsize=0.8       ;size of the text
  pcharthick=1        ;thickness of the text
  color_active=[255, 0, 0]      ;color of text, when window is active [r, g, b]
  color_inactive=[5, 180, 150]  ;color of text, when window is inactive [r, g, b]
  
  ;sizes of CCDs
  PixCCDxfuv1=2072 ;number of pixels on FUV1 CCD in x-direction
  PixCCDyfuv1=1096 ;number of pixels on FUV1 CCD in y-direction
  PixCCDxfuv2=PixCCDxfuv1 ;number of pixels on FUV2 CCD in x-direction
  PixCCDyfuv2=PixCCDyfuv1 ;number of pixels on FUV2 CCD in y-direction
  PixCCDxnuv=PixCCDxfuv1 ;number of pixels on NUV CCD in x-direction
  PixCCDynuv=PixCCDyfuv1 ;number of pixels on NUV CCD in y-direction
  PixCCDxfuvSJI=PixCCDxfuv1/2 ;number of pixels on FUV SJI CCD in x-direction
  PixCCDyfuvSJI=PixCCDyfuv1 ;number of pixels on FUV SJI CCD in y-direction
  PixCCDxnuvSJI=PixCCDxfuv1/2 ;number of pixels on NUV SJI CCD in x-direction
  PixCCDynuvSJI=PixCCDyfuv1 ;number of pixels on NUV SJI CCD in y-direction
  PZTnoChangeValue=9999     ; if all PZT variables in OBS have this value, the position is not changed
  
  
  
  
  ; get current date and time to create a new folder
  if savefitsfiles || saveplot || makemovie || savelogcsv || savelogtxt || copyxml then begin
    datenum=bin_date(systime(/utc))
    datestamp=''
    for i=0,5 do begin
      if datenum[i] lt 10 then datestamp=datestamp + '0'
      datestamp = datestamp + strcompress(string(datenum[i]),/remove_all)
      if i lt 5 then datestamp = datestamp + '_'
    endfor
    datestamp=datestamp+'_L1'
    print, 'saving data in folder: ', datestamp
    spawn, "mkdir "+datestamp
  endif
  
  ;get data from XML-files and save the xml-files to the above created folder
  readtables, OBSfile, OBSdir, OBSList, FRMList, FDBList, CRSList, error, copyxml, datestamp
  
  ; check for errors while reading the XML-files
  if error then begin
    print,'There is something wrong with the given OBS file, exiting program'
    return
  endif
  
  
  ; get list of input files
  fuvfileList = find_file(inputdirFUV+inputfileFUV)
  if (size(fuvfileList))[0] eq 0 then begin
    print, 'No FUV input files found'
    return
  endif
  if (size(fuvfileList))[1] eq 1 then begin
    print, 'Found only 1 FUV input file'
    return
  endif
  f=file_basename(fuvfileList)
  ff=strmid(f,stregex(f[0],'[1-9][0-9][0-9]'),3)
  fuvfilenumbers=uint(ff)
  
  nuvfileList = find_file(inputdirNUV+inputfileNUV)
  if (size(nuvfileList))[0] eq 0 then begin
    print, 'No NUV input files found'
    return
  endif
  if (size(nuvfileList))[1] eq 1 then begin
    print, 'Found only 1 NUV input file'
    return
  endif
  f=file_basename(nuvfileList)
  ff=strmid(f,stregex(f[0],'[1-9][0-9][0-9]'),3)
  nuvfilenumbers=uint(ff)
  
  
  
  
  
  
  
  for nrOBSrepeats=0,OBSList.Repeat_Obs-1 do begin ; loop over OBS repeats
    print, 'OBS repeat ', nrOBSrepeats+1, ' of ', OBSList.Repeat_Obs
    
    if nrOBSrepeats eq 0 then begin
      ;initialize time and some other values
      time = OBSList.Tref_Obs
      localtimefuv = time
      localtimenuv = time
      timeOBSrep = time
      OBScadence = OBSList.Cadence_Obs
      PZT = [0d, 0d]
      step = 0
    endif else begin
      ;add cadence of OBS to time and check if cadence is not too short
      timeOBSrep = timeOBSrep + OBScadence
      if timeOBSrep gt time then begin
        localtimefuv = localtimefuv + timeOBSrep - time
        localtimenuv = localtimenuv + timeOBSrep - time
        time = timeOBSrep
      endif
    endelse
    
    
    for nrFRM=0,OBSList.NumEntries-1 do begin ;loop over whole OBS, different FRMs
    
      if (*(OBSList).Tref_FRM)[nrFRM] gt time then begin
        ; check if time is correct, or has to be adjusted
        localtimefuv = localtimefuv + (*(OBSList).Tref_FRM)[nrFRM] - time
        localtimenuv = localtimenuv + (*(OBSList).Tref_FRM)[nrFRM] - time
        time = (*(OBSList).Tref_FRM)[nrFRM]
      endif
      
      ; set position of lower left corner of origin, according to PZT
      if (*(OBSList).PZT_A_Abs)[nrFRM] ne PZTnoChangeValue || $
        (*(OBSList).PZT_B_Abs)[nrFRM] ne PZTnoChangeValue || $
        (*(OBSList).PZT_C_Abs)[nrFRM] ne PZTnoChangeValue then begin
        PZT = pzt2xy((*(OBSList).PZT_A_Abs)[nrFRM], (*(OBSList).PZT_B_Abs)[nrFRM], (*(OBSList).PZT_C_Abs)[nrFRM])
        ; PZT = round(PZT*6)
        addstep=0
      endif else begin
        if N_ELEMENTS(PZTStep) ne 0 then PZT = PZT-PZTStep ;reverse last addition of step, because we want to add the current stepsize
        addstep=1
      endelse
      PZTStep = pzt2xy((*(OBSList).PZT_A_Step)[nrFRM], (*(OBSList).PZT_B_Step)[nrFRM], (*(OBSList).PZT_C_Step)[nrFRM])
      ; PZTStep = round(PZTStep*6)
      if addstep then PZT = PZT+PZTStep
      
      ;get current FRM index
      currentFRM=-1
      for i=0,N_ELEMENTS(FRMList)-1 do begin
        if (*(OBSList).FRM_ID)[nrFRM] eq FRMList[i].ID then begin
          currentFRM = i
          break
        endif
      endfor
      if currentFRM eq -1 then begin
        print, 'Could not find corresponding FRM ', (*(OBSList).FRM_ID)[nrFRM]
        return
      endif
      print, 'current FRM ID: ', FRMList[currentFRM].ID
      
      
      
      for nrFRMrepeats=0,(*(OBSList).Repeat_FRM)[nrFRM]-1 do begin ; loop over FRM repeats
      
        if nrFRMrepeats eq 0 then begin
          ;get cadence of 1 FRM (cadence given in FRM-file is for all repeats)
          timeFRMrep = time
          FRMcadence = (*(OBSList).Cadence_FRM)[nrFRM]
        endif
        
        
        for nrFDB=0,FRMList[currentFRM].NumEntries-1 do begin ; loop over current FRM, different entries
        
          if (*(FRMList[currentFRM]).Tref)[nrFDB] gt 0 then begin
            ;check time given within FRM-file, and add it to the time
            if nrFDB gt 0 then subtract = (*(FRMList[currentFRM]).Tref)[nrFDB-1] else subtract = 0
            time = time + (*(FRMList[currentFRM]).Tref)[nrFDB] - subtract
            localtimefuv = localtimefuv + (*(FRMList[currentFRM]).Tref)[nrFDB] - subtract
            localtimenuv = localtimenuv + (*(FRMList[currentFRM]).Tref)[nrFDB] - subtract
          endif
          
          PZTOFF = pzt2xy((*(FRMList[currentFRM]).PZT_A_Rel)[nrFDB], (*(FRMList[currentFRM]).PZT_B_Rel)[nrFDB], (*(FRMList[currentFRM]).PZT_C_Rel)[nrFDB])
          ; PZTOFF = round(PZTOFF*6)
          PZTcurrent = PZT + PZTOFF
          
          ;get current FDB indices
          currentFDB=[-1, -1, -1] ;[sji, nuv, fuv]
          noFDB=0
          if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          if (*(FRMList[currentFRM]).FUV_SG_FDB_ID)[nrFDB] eq '0' then noFDB = noFDB + 1
          for i=0,N_ELEMENTS(FDBList)-1 do begin
            if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).SJI_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[0] = i
            if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).NUV_SG_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[1] = i
            if (*(FRMList[currentFRM]).FUV_SG_FDB_ID)[nrFDB] ne '0' then $
              if (*(FRMList[currentFRM]).FUV_SG_FDB_ID)[nrFDB] eq FDBList[i].ID then currentFDB[2] = i
          endfor
          index = WHERE(currentFDB eq -1, count)
          if (count eq 3) || (count ne noFDB) then begin
            print, 'Could not find one or more corresponding FDB in this FRM ', FRMList[currentFRM].ID
            return
          endif
          ;print, 'current FDB indices: ', currentFDB
          
          
          ;get current CRS indices
          currentCRS=[-1, -1, -1] ;[sji, nuv, fuv]
          for i=0,N_ELEMENTS(CRSList)-1 do begin
            for j=0,2 do begin
              if currentFDB[j] ge 0 then $
                if FDBList[currentFDB[j]].CRSID eq CRSList[i].ID then currentCRS[j] = i
            endfor
          endfor
          index = WHERE(currentCRS eq -1, count)
          if (count eq 3) || (count ne noFDB) then begin
            print, 'Could not find one or more corresponding CRS'
            return
          endif
          ;print, 'current CRS indices: ', currentCRS
          
          
          
          if step eq 0 then begin
            ;initial values, files
            ;FUV
            fuvfileind1=0
            fuvfileind2=1
            dtimeSIMfuv = dtimeSIM * (fuvfilenumbers[fuvfileind2]-fuvfilenumbers[fuvfileind1])
            while localtimefuv gt dtimeSIMfuv do begin
              localtimefuv = localtimefuv - dtimeSIMfuv
              fuvfileind1=fuvfileind2
              fuvfileind2=fuvfileind2+1
              if fuvfileind2 eq N_ELEMENTS(fuvfilenumbers) then begin
                fuvfileind2 = 0
                dtimeSIMfuv = dtimeSIMend
              endif else begin
                dtimeSIMfuv = dtimeSIM * (fuvfilenumbers[fuvfileind2]-fuvfilenumbers[fuvfileind1])
              endelse
            endwhile
            print, 'loading new data file: ', fuvfileList[fuvfileind1]
            sim1=fuvfilenumbers[fuvfileind1]
            simfile1=file_basename(fuvfileList[fuvfileind1])
            filedirfuv=inputdirFUV+inputfileFUV
            fxread, fuvfileList[fuvfileind1], fuvSJI1, header
            fxbopen,unit,fuvfileList[fuvfileind1],1,header
            fxbread,unit,fuv11,'FUV 1'
            fxbread,unit,fuv12,'FUV 2'
            fxbclose,unit
            
            print, 'loading new data file: ', fuvfileList[fuvfileind2]
            sim2=fuvfilenumbers[fuvfileind2]
            simfile2=file_basename(fuvfileList[fuvfileind2])
            fxread, fuvfileList[fuvfileind2], fuvSJI2, header
            fxbopen,unit,fuvfileList[fuvfileind2],1,header
            fxbread,unit,fuv21,'FUV 1'
            fxbread,unit,fuv22,'FUV 2'
            fxbclose,unit
            
            ;NUV
            nuvfileind1=0
            nuvfileind2=1
            dtimeSIMnuv = dtimeSIM * (nuvfilenumbers[nuvfileind2]-nuvfilenumbers[nuvfileind1])
            while localtimenuv gt dtimeSIMnuv do begin
              localtimenuv = localtimenuv - dtimeSIMnuv
              nuvfileind1=nuvfileind2
              nuvfileind2=nuvfileind2+1
              if nuvfileind2 eq N_ELEMENTS(nuvfilenumbers) then begin
                nuvfileind2 = 0
                dtimeSIMnuv = dtimeSIMend
              endif else begin
                dtimeSIMnuv = dtimeSIM * (nuvfilenumbers[nuvfileind2]-nuvfilenumbers[nuvfileind1])
              endelse
            endwhile
            print, 'loading new data file: ', nuvfileList[nuvfileind1]
            sim3=nuvfilenumbers[nuvfileind1]
            simfile3=file_basename(nuvfileList[nuvfileind1])
            filedirnuv=inputdirNUV+inputfileNUV
            nuvSJI1 = readfits(nuvfileList[nuvfileind1], header2, /silent)
            nuv1lambda = readfits(nuvfileList[nuvfileind1], header21, exten_no=1, /silent)
            nuv1 = readfits(nuvfileList[nuvfileind1], header22, exten_no=2, /silent)
            
            print, 'loading new data file: ', nuvfileList[nuvfileind2]
            sim4=nuvfilenumbers[nuvfileind2]
            simfile4=file_basename(nuvfileList[nuvfileind2])
            nuvSJI2 = readfits(nuvfileList[nuvfileind2], header2, /silent)
            nuv2lambda = readfits(nuvfileList[nuvfileind2], header21, exten_no=1, /silent)
            nuv2 = readfits(nuvfileList[nuvfileind2], header22, exten_no=2, /silent)
            
          endif else begin
            ;check if we need to load a new simulation file for FUV
            ;FUV
            if localtimefuv ge dtimeSIMfuv then begin
              localtimefuv = localtimefuv-dtimeSIMfuv
              fuvfileind1=fuvfileind2
              fuvfileind2=fuvfileind2+1
              if fuvfileind2 eq N_ELEMENTS(fuvfilenumbers) then begin
                fuvfileind2 = 0
                dtimeSIMfuv = dtimeSIMend
              endif else begin
                dtimeSIMfuv = dtimeSIM * (fuvfilenumbers[fuvfileind2]-fuvfilenumbers[fuvfileind1])
              endelse
              load2=0
              while localtimefuv gt dtimeSIMfuv do begin
                load2=1
                localtimefuv = localtimefuv - dtimeSIMfuv
                fuvfileind1=fuvfileind2
                fuvfileind2=fuvfileind2+1
                if fuvfileind2 eq N_ELEMENTS(fuvfilenumbers) then begin
                  fuvfileind2 = 0
                  dtimeSIMfuv = dtimeSIMend
                endif else begin
                  dtimeSIMfuv = dtimeSIM * (fuvfilenumbers[fuvfileind2]-fuvfilenumbers[fuvfileind1])
                endelse
              endwhile
              
              if load2 then begin ;time step is too big, we need to skip a file (or more)
                print, 'loading new data file: ', fuvfileList[fuvfileind1]
                sim1=fuvfilenumbers[fuvfileind1]
                simfile1=file_basename(fuvfileList[fuvfileind1])
                fxread, fuvfileList[fuvfileind1], fuvSJI1, header
                fxbopen,unit,fuvfileList[fuvfileind1],1,header
                fxbread,unit,fuv11,'FUV 1'
                fxbread,unit,fuv12,'FUV 2'
                fxbclose,unit
              endif else begin
                sim1=sim2
                simfile1=simfile2
                fuvSJI1=fuvSJI2
                fuv11=fuv21
                fuv12=fuv22
              endelse
              print, 'loading new data file: ', fuvfileList[fuvfileind2]
              sim2=fuvfilenumbers[fuvfileind2]
              simfile2=file_basename(fuvfileList[fuvfileind2])
              fxread, fuvfileList[fuvfileind2], fuvSJI2, header
              fxbopen,unit,fuvfileList[fuvfileind2],1,header
              fxbread,unit,fuv21,'FUV 1'
              fxbread,unit,fuv22,'FUV 2'
              fxbclose,unit
            endif ;FUV
            
            
            if localtimenuv ge dtimeSIMnuv then begin
              ;NUV
              localtimenuv = localtimenuv-dtimeSIMnuv
              nuvfileind1=nuvfileind2
              nuvfileind2=nuvfileind2+1
              if nuvfileind2 eq N_ELEMENTS(nuvfilenumbers) then begin
                nuvfileind2 = 0
                dtimeSIMnuv = dtimeSIMend
              endif else begin
                dtimeSIMnuv = dtimeSIM * (nuvfilenumbers[nuvfileind2]-nuvfilenumbers[nuvfileind1])
              endelse
              load2=0
              while localtimenuv gt dtimeSIMnuv do begin
                load2=1
                localtimenuv = localtimenuv - dtimeSIMnuv
                nuvfileind1=nuvfileind2
                nuvfileind2=nuvfileind2+1
                if nuvfileind2 eq N_ELEMENTS(nuvfilenumbers) then begin
                  nuvfileind2 = 0
                  dtimeSIMnuv = dtimeSIMend
                endif else begin
                  dtimeSIMnuv = dtimeSIM * (nuvfilenumbers[nuvfileind2]-nuvfilenumbers[nuvfileind1])
                endelse
              endwhile
              
              if load2 then begin ;time step is too big, we need to skip a file (or more)
                print, 'loading new data file: ', nuvfileList[nuvfileind1]
                sim3=nuvfilenumbers[nuvfileind1]
                simfile3=file_basename(nuvfileList[nuvfileind1])
                nuvSJI1 = readfits(nuvfileList[nuvfileind1], header2, /silent)
                nuv1lambda = readfits(nuvfileList[nuvfileind1], header21, exten_no=1, /silent)
                nuv1 = readfits(nuvfileList[nuvfileind1], header22, exten_no=2, /silent)
              endif else begin
                sim3=sim4
                simfile3=simfile4
                nuvSJI1=nuvSJI2
                nuv1=nuv2
                nuv1lambda=nuv2lambda
              endelse
              print, 'loading new data file: ', nuvfileList[nuvfileind2]
              sim4=nuvfilenumbers[nuvfileind2]
              simfile4=file_basename(nuvfileList[nuvfileind2])
              nuvSJI2 = readfits(nuvfileList[nuvfileind2], header2, /silent)
              nuv2lambda = readfits(nuvfileList[nuvfileind2], header21, exten_no=1, /silent)
              nuv2 = readfits(nuvfileList[nuvfileind2], header22, exten_no=2, /silent)
            endif
          endelse
          
          
          if step eq 0 then begin
            ; initialize window parameters
            !p.charsize=pcharsize
            !p.charthick=pcharthick
            
            initGraph, OBSList, FRMList, FDBList, CRSList, round(SJIframe*6), PZTnoChangeValue, PixCCDxfuv1, PixCCDxfuvSJI, $
              SpecData, SJIData, rasterSJI, WINsize, PosClock, xmax=maxWINsize[0], ymax=maxWINsize[1]
              
            frozenSlit=intarr(N_ELEMENTS(SJIData))
            
            ;create logfiles
            if savelogcsv || savelogtxt then begin
              writeLogFile, savelogcsv, savelogtxt, logFilecsvName, logFiletxtName, csvUnit, txtUnit, 1, $
                filedirfuv=filedirfuv, filedirnuv=filedirnuv, datestamp=datestamp, OBSid=OBSList.ID
            endif
            
          endif
          
          ;write to the csv-log file
          if savelogcsv || savelogtxt then begin
            if currentFDB[0] ge 0 then fdbsji=FDBList[currentFDB[0]].ID else fdbsji='-'
            if currentFDB[2] ge 0 then fdbfuv=FDBList[currentFDB[2]].ID else fdbfuv='-'
            if currentFDB[1] ge 0 then fdbnuv=FDBList[currentFDB[1]].ID else fdbnuv='-'
            if currentCRS[0] ge 0 then crssji=CRSList[currentCRS[0]].ID else crssji='-'
            if currentCRS[2] ge 0 then crsfuv=CRSList[currentCRS[2]].ID else crsfuv='-'
            if currentCRS[1] ge 0 then crsnuv=CRSList[currentCRS[1]].ID else crsnuv='-'
            writeLogFile, savelogcsv, savelogtxt, logFilecsvName, logFiletxtName, csvUnit, txtUnit, 0, 0, $
              time, fdbsji, fdbfuv, fdbnuv, crssji, crsfuv, crsnuv, $
              PZTcurrent, nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
              (*(FRMList[currentFRM]).FW)[nrFDB], FRMList[currentFRM].ID, $
              sim1, sim2, sim3, sim4
          endif
          
          
          
          
          ;timefactor for interpolation between time steps
          timefacfuv = double(localtimefuv)/dtimeSIMfuv
          timefacnuv = double(localtimenuv)/dtimeSIMnuv
          
          SpecwinUpdate=intarr(N_ELEMENTS(SpecData))
          
          ;calculate FUV if necessary
          if currentCRS[2] ge 0 then begin
            ;interpolation in time, x- and y direction
            fuv1 = DataPrepareSpec(fuv11, fuv21, PixCCDxfuv1, PixCCDyfuv1, PZTcurrent, timefacfuv, /rightpad)
            fuv2 = DataPrepareSpec(fuv12, fuv22, PixCCDxfuv2, PixCCDyfuv2, PZTcurrent, timefacfuv)
            ;put fuv1 and fuv2 into 1 matrix
            fuv = fltarr((size(fuv1))[1]+(size(fuv2))[1], max([(size(fuv1))[2], (size(fuv2))[2]]))
            fuv[0:(size(fuv1))[1]-1, 0: (size(fuv1))[2]-1] = fuv1
            fuv[(size(fuv1))[1]:(size(fuv1))[1]+(size(fuv2))[1]-1, 0:(size(fuv2))[2]-1] = fuv2
            
            ;check if we need to rebin the data
            sumspec=CRSList[currentCRS[2]].Spectral
            sumspat=CRSList[currentCRS[2]].Spatial
            if (sumspec gt 1) || (sumspat gt 1) then $
              fuv = rebin(fuv, (PixCCDxfuv1+PixCCDxfuv2)/sumspec, max([PixCCDyfuv1, PixCCDyfuv2])/sumspat) * sumspec * sumspat
              
            ;multiply with defined exposure duratioin
            fuv = fuv * FDBList[currentFDB[2]].Exp_Duration/1000d
            
            ; set all data to zero, except for CRS windows
            fuvtemp = fltarr((size(fuv))[1],(size(fuv))[2])
            for i=0,CRSList[currentCRS[2]].SubRegions-1 do begin
              for pda=0,N_ELEMENTS(SpecData)-1 do begin
                if (SpecData[pda].CRSindex eq currentCRS[2]) && (SpecData[pda].SubRegionID eq (*(CRSList[currentCRS[2]]).SubRegionID)[i]) then begin
                  startR = SpecData[pda].SIMcoords[0]
                  endR = SpecData[pda].SIMcoords[4]
                  startC = SpecData[pda].SIMcoords[2]
                  endC = SpecData[pda].SIMcoords[5]
                  ;adjust for rebinning
                  if sumspec gt 1 then begin
                    startR = startR / sumspec
                    endR = endR / sumspec
                  endif
                  if sumspat gt 1 then begin
                    startC = startC / sumspat
                    endC = endC / sumspat
                  endif
                  fuvtemp[startR:endR,startC:endC] = fuv[startR:endR,startC:endC]
                  
                  ;set data of region to structure SpecData for later plotting
                  *(SpecData[pda]).data = fuv[startR:endR,startC:endC]
                  SpecwinUpdate[pda]=1
                  
                  ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
                  if (SpecData[pda].min_show eq 0) && (SpecData[pda].max_show eq 0) then begin
                    temp = Histo_Opt(*(SpecData[pda]).data, threshold_Spec)
                    SpecData[pda].max_show = max(temp, min=m)
                    SpecData[pda].min_show = m
                  endif
                  
                  ;we save the PZT offset for each SJI indivually, for the axis of the plot
                  SpecData[pda].PZTx = PZTcurrent[0]
                  SpecData[pda].PZTy = PZTcurrent[1]
                  
                  break
                endif
              endfor
            endfor
            fuv = fuvtemp
            
          endif ;FUV calculation
          
          
          
          ;calculate NUV if necessary
          if currentCRS[1] ge 0 then begin
            ;interpolation in time, x- and y direction
            nuv = DataPrepareSpec(nuv1, nuv2, PixCCDxnuv, PixCCDynuv, PZTcurrent, timefacnuv, /rightpad, /nuv)
            
            ;check if we need to rebin the data
            sumspec=CRSList[currentCRS[1]].Spectral
            sumspat=CRSList[currentCRS[1]].Spatial
            if (sumspec gt 1) || (sumspat gt 1) then $
              nuv = rebin(nuv, PixCCDxnuv/sumspec, PixCCDynuv/sumspat) * sumspec * sumspat
              
            ;multiply with defined exposure duration
            nuv = nuv * FDBList[currentFDB[1]].Exp_Duration/1000d
            
            ; set all data to zero, except for CRS windows
            nuvtemp = fltarr((size(nuv))[1],(size(nuv))[2])
            for i=0,CRSList[currentCRS[1]].SubRegions-1 do begin
              for pda=0,N_ELEMENTS(SpecData)-1 do begin
                if (SpecData[pda].CRSindex eq currentCRS[1]) && (SpecData[pda].SubRegionID eq (*(CRSList[currentCRS[1]]).SubRegionID)[i]) then begin
                  startR = SpecData[pda].SIMcoords[0]
                  if startR ge PixCCDxfuv1 then startR = startR - PixCCDxfuv1
                  endR = SpecData[pda].SIMcoords[4]
                  if endR ge PixCCDxfuv1 then endR = endR - PixCCDxfuv1
                  startC = SpecData[pda].SIMcoords[2]
                  endC = SpecData[pda].SIMcoords[5]
                  ;adjust for rebinning
                  if sumspec gt 1 then begin
                    startR = startR / sumspec
                    endR = endR / sumspec
                  endif
                  if sumspat gt 1 then begin
                    startC = startC / sumspat
                    endC = endC / sumspat
                  endif
                  nuvtemp[startR:endR,startC:endC] = nuv[startR:endR,startC:endC]
                  
                  ;set data of region to structure PlotData for later plotting
                  *(SpecData[pda]).data = nuv[startR:endR,startC:endC]
                  SpecwinUpdate[pda]=1
                  
                  ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
                  if (SpecData[pda].min_show eq 0) && (SpecData[pda].max_show eq 0) then begin
                    temp = Histo_Opt(*(SpecData[pda]).data, threshold_Spec)
                    SpecData[pda].max_show = max(temp, min=m)
                    SpecData[pda].min_show = m
                  endif
                  
                  ;we save the PZT offset for each SJI indivually, for the axis of the plot
                  SpecData[pda].PZTx = PZTcurrent[0]
                  SpecData[pda].PZTy = PZTcurrent[1]
                  
                  break
                endif
              endfor
            endfor
            nuv = nuvtemp
            
          endif ;NUV calculation
          
          
          
          ;SJI calculation
          SJIwinUpdate=intarr(N_ELEMENTS(SJIData)) ;indicates whether a specific SJI window is updated
          SJIwinUpdate[0]=1 ;Overview SJI window is always updated
          fuvSlit=0
          if currentCRS[0] ge 0 then begin
            for i=1,N_ELEMENTS(SJIData)-1 do begin
              if SJIData[i].CRSindex eq currentCRS[0] then begin
                SJIwinUpdate[i]=1
                frozenSlit[i]=PixCCDxfuvSJI/2
                if SJIData[i].type eq 'fuv' then frozenSlit[i]=frozenSlit[i]+PixCCDxfuvSJI
                if SJIData[i].type eq 'both' then begin
                  if ((*(FRMList[currentFRM]).FW)[nrFDB] eq 31) || ((*(FRMList[currentFRM]).FW)[nrFDB] eq 91) then begin
                    frozenSlit[i]=frozenSlit[i]+PixCCDxfuvSJI
                    fuvSlit=PixCCDxfuvSJI
                  endif
                endif
                if SJIData[i].sum_spatX gt 1 then $
                  frozenSlit[i]=SJIData[i].Boxcoords[0]+1+(frozenSlit[i]-SJIData[i].Boxcoords[0]-1)/SJIData[i].sum_spatX
              endif
            endfor
          endif
          DataPrepareSJI, fuvSJI1, fuvSJI2, nuvSJI1, nuvSJI2, rasterSJI, PixCCDxfuvSJI, PixCCDyfuvSJI, timefacfuv, timefacnuv, PZTcurrent, SJIData, $
            FRMList[currentFRM], nrFDB, FDBList[currentFDB[0]], CRSList[currentCRS[0]], SJI, SJIwinUpdate, $
            threshold_SJI_Overview, threshold_SJI
          currentBox=SJIData[0].Showcoords
          currentBox[0]=currentBox[0]+PZTcurrent[0]*6
          currentBox[4]=currentBox[4]+PZTcurrent[0]*6
          currentBox[2]=currentBox[2]+PZTcurrent[1]*6
          currentBox[5]=currentBox[5]+PZTcurrent[1]*6
          currentSlit=PixCCDxfuvSJI/2+PZTcurrent[0]*6+fuvSlit
          ;currentMiddle=PixCCDyfuvSJI/2+PZTcurrent[1]*6
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          if savefitsfiles then begin
          
            if currentCRS[0] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_SJI_L1.fits'
              fxhmake, header, SJI, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, SpecData, $
                0, currentFRM, currentFDB[0], currentCRS[0], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                simfile1, simfile2, simfile3, simfile4
              fxwrite, file, header, SJI
            endif
            
            if currentCRS[1] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_NUV_L1.fits'
              fxhmake, header, nuv, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, SpecData, $
                1, currentFRM, currentFDB[1], currentCRS[1], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                simfile1, simfile2, simfile3, simfile4
              fxwrite, file, header, nuv
            endif
            
            if currentCRS[2] ge 0 then begin
              file=datestamp + fns('/AIA_####',step) + '_FUV_L1.fits'
              ; Create a primary header and write it out
              fxhmake, header, fuv, /initialize, /extend, /date
              SyntoolAddKeywords, header, OBSList, FRMList, FDBList, CRSList, SpecData, $
                2, currentFRM, currentFDB[2], currentCRS[2], $
                nrOBSrepeats, nrFRM, nrFRMrepeats, nrFDB, $
                time, PZT, $
                simfile1, simfile2, simfile3, simfile4
              fxwrite, file, header, fuv
            endif
            
          endif ;save fits files
          
          
          
          
          
          ;;; graphical output
          
          if step eq 0 then begin
            if saveplot then set_plot,'z' else set_plot,'x'
            !p.background=253
            !p.color=0
            !p.ticklen=-.02
            ;!p.charsize=1  ;those 2 are defined above
            ;!p.charthick=1
            if saveplot then device,set_res=WINsize $
            else window, 16, xs=WINsize[0], ys=WINsize[1]
            ;load color table, works only with set_plot,'x', or see 3 lines below
            loadct,0
            ;reverse black and white, works like this only when plotting directly on screen, not in file
            ;but like this it works: tvrd(),r,g,b
            tvlct,r,g,b,/get
            ;r=reverse(r)
            ;g=reverse(g)
            ;b=reverse(b)
            ;active color
            r[255]=color_active[0]
            g[255]=color_active[1]
            b[255]=color_active[2]
            ;non-active color
            r[254]=color_inactive[0]
            g[254]=color_inactive[1]
            b[254]=color_inactive[2]
            tvlct,r,g,b
          endif
          
          
          
          
          ;plot SJI
          for i=0,N_ELEMENTS(SJIData)-1 do begin
            if SJIwinUpdate[i] then colortext=255 else colortext=254
            ;print,i,SJIData[i].Boxcoords
            bold=1
            titlesize=1.0*!p.charsize
            titleposy=titlesize*!d.y_ch_size/WINsize[1]
            datasize=0.9d*!p.CHARSIZE
            dataposy=datasize*!d.y_ch_size/WINsize[1]
            leftpos=0.1*!d.x_ch_size/WINsize[0]
            
            if SJIData[i].min_show eq SJIData[i].max_show then maxshow=SJIData[i].max_show+1e-10 else maxshow=SJIData[i].max_show
            
            if i eq 0 then begin
              ;Overview SJI
              pih, *(SJIData[i]).data, threshold_SJI_Overview, position=SJIData[i].WINcoords, $
                origin=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[2]+1], $
                min=SJIData[i].min_show, max=maxshow, $
                xstyle=5, ystyle=5, top=253
              ;axis, xaxis=1, xrange=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[4]+1], xtit='Solar X [Px]', xstyle=1
              axis, xaxis=0, xrange=[(-SJIData[i].Boxcoords[1])/2/6.0, (SJIData[i].Boxcoords[1])/2/6.0], xtit='Solar X [arcsec]', xstyle=1
              ;axis, yaxis=1, yrange=[SJIData[i].Boxcoords[2]+1, SJIData[i].Boxcoords[5]+1], ytit='Solar Y [Px]', ystyle=1
              axis, yaxis=0, yrange=[(-SJIData[i].Boxcoords[3])/2/6.0, (SJIData[i].Boxcoords[3])/2/6.0], ytit='Solar Y [arcsec]', ystyle=1
              oplot, [currentBox[0], currentBox[0]], [currentBox[2], currentBox[5]], thick=2, col=128
              oplot, [currentBox[4], currentBox[4]], [currentBox[2], currentBox[5]], thick=2, col=128
              oplot, [currentBox[0], currentBox[4]], [currentBox[2], currentBox[2]], thick=2, col=128
              oplot, [currentBox[0], currentBox[4]], [currentBox[5], currentBox[5]], thick=2, col=128
              oplot, [currentSlit, currentSlit], [SJIData[i].Showcoords[2]+PZTcurrent[1]*6, SJIData[i].Showcoords[5]+PZTcurrent[1]*6], thick=2, col=128
            endif else begin
              pih,*(SJIData[i]).data, threshold_SJI, position=SJIData[i].WINcoords, $
                origin=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[2]+1], $
                min=SJIData[i].min_show, max=maxshow, $
                xstyle=5, ystyle=5, /noerase, top=253
                
              ;upper x-axis
              ;if SJIData[i].type eq 'fuv' then shiftpx=PixCCDxnuvSJI else shiftPx=0
              axis,xaxis=1,xrange=[SJIData[i].Boxcoords[0]+1, SJIData[i].Boxcoords[4]+1], $
                xtit='Solar X [Px]', xstyle=1, xticks=SJIData[i].nrticksx, xtickformat='(i4)'
              ;lower x-axis
              if SJIData[i].Boxcoords[0] ge PixCCDxfuvSJI then shiftxarc=3. else shiftxarc=1.
              axis,xaxis=0,xrange=[((SJIData[i].Boxcoords[0]+1-PixCCDxfuvSJI*shiftxarc/2))/6.0+SJIData[i].PZTx, ((SJIData[i].Boxcoords[4]+1-PixCCDxfuvSJI*shiftxarc/2))/6.0+SJIData[i].PZTx], $
                xtit='Solar X [arcsec]', xstyle=1, xticks=SJIData[i].nrticksx, xtickformat='(i4)'
              ;right y-axis
              if SJIData[i].axisR then $
                axis,yaxis=1,yrange=[SJIData[i].Boxcoords[2]+1, SJIData[i].Boxcoords[5]+1], $
                ytit='Solar Y [Px]', ystyle=1, yticks=SJIData[i].nrticksy, ytickformat='(i4)'
              ;left y-axis
              if SJIData[i].axisL then $
                axis,yaxis=0,yrange=[(SJIData[i].Boxcoords[2]+1-PixCCDynuvSJI/2)/6.0+SJIData[i].PZTy, (SJIData[i].Boxcoords[5]+1-PixCCDynuvSJI/2)/6.0+SJIData[i].PZTy], $
                ytit='Solar Y [arcsec]', ystyle=1, yticks=SJIData[i].nrticksy, ytickformat='(i4)'
            endelse
            
            
            ;draw slit if necessary
            if frozenSlit[i] gt 0 then begin
              oplot, [frozenSlit[i], frozenSlit[i]], [SJIData[i].BOXcoords[2], SJIData[i].BOXcoords[5]], thick=2, col=128
            endif
            
            ;write title and some values
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[3]-1.2*titleposy, SJIData[i].name, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[3]-2.4*titleposy, SJIData[i].name2, color=colortext, charsize=titlesize, charthick=bold, /normal
            if i eq 0 then begin
              text1='X '+string(PZTcurrent[0],format='(f6.2)')
              text2='Y '+string(PZTcurrent[1],format='(f6.2)')
            endif else begin
              maxval=string(max(*(SJIData[i]).data),format='(e7.1)')
              meanval=string(mean(*(SJIData[i]).data),format='(e7.1)')
              text1='A '+meanval
              text2='M '+maxval
            endelse
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[1]+1.5*dataposy, text1, color=colortext, charsize=datasize, charthick=bold, /normal
            xyouts, SJIData[i].WINcoords[0]+leftpos, SJIData[i].WINcoords[1]+0.3*dataposy, text2, color=colortext, charsize=datasize, charthick=bold, /normal
          endfor
          
          
          
          
          
          
          
          
          
          
          
          
          
          
          ;plot Spectra
          for i=0,N_ELEMENTS(SpecData)-1 do begin
            if SpecwinUpdate[i] then colortext=255 else colortext=254
            maxval=string(max(*(SpecData[i]).data),format='(e7.1)')
            meanval=string(mean(*(SpecData[i]).data),format='(e7.1)')
            if SpecData[i].min_show eq SpecData[i].max_show then maxshow=SpecData[i].max_show+1e-10 else maxshow=SpecData[i].max_show
            case SpecData[i].type of
              'fu1': begin
                displambda=0.01298
                startlambda=1331.6
              end
              'fu2': begin
                displambda=0.01272
                startlambda=1380.6 - displambda * PixCCDxfuv1
              end
              'nuv': begin
                displambda=0.02546
                startlambda=2782.6 - displambda * PixCCDxfuv1
              end
            endcase
            pih,*(SpecData[i]).data*(-1d), threshold_Spec, position=SpecData[i].WINcoords, $
              max=SpecData[i].min_show*(-1d), min=maxshow*(-1d), $
              xstyle=5, ystyle=5, $
              /noerase, top=253;, noscale=1
            ;upper x-axis
            axis, xaxis=1, xrange=[SpecData[i].SIMcoords[0]+1, SpecData[i].SIMcoords[4]+1], $
              xtit=Greek('lambda')+' [Px]', xstyle=1, xticks=SpecData[i].nrticksx, xtickformat='(i4)'
            ;lower x-axis
            axis, xaxis=0, xrange=[startlambda+displambda*SpecData[i].SIMcoords[0], startlambda+displambda*SpecData[i].SIMcoords[4]], $
              xtit=Greek('lambda')+' [!3' + String("305B) + '!X]', xstyle=1, xticks=SpecData[i].nrticksx, xtickformat='(i4)'
            ;right y-axis
            if SpecData[i].axisR then $
              axis, yaxis=1, yrange=[SpecData[i].SIMcoords[2]+1, SpecData[i].SIMcoords[5]+1], $
              ytit='Solar Y [Px]', ystyle=1, yticks=SpecData[i].nrticksy, ytickformat='(i4)'
            ;left y-axis
            if SpecData[i].axisL then $
              axis, yaxis=0, yrange=[(SpecData[i].SIMcoords[2]+1-PixCCDynuv/2)/6.0+SpecData[i].PZTy, (SpecData[i].SIMcoords[5]+1-PixCCDynuv/2)/6.0+SpecData[i].PZTy], $
              ytit='Solar Y [arcsec]', ystyle=1, yticks=SpecData[i].nrticksy, ytickformat='(i4)'
              
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[3]-1.2*titleposy, SpecData[i].name, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[3]-2.4*titleposy, SpecData[i].name2, color=colortext, charsize=titlesize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[1]+1.5*dataposy, 'A '+meanval, color=colortext, charsize=datasize, charthick=bold, /normal
            xyouts, SpecData[i].WINcoords[0]+leftpos, SpecData[i].WINcoords[1]+0.3*dataposy, 'M '+maxval, color=colortext, charsize=datasize, charthick=bold, /normal
          endfor
          
          
          ;display time, digitally and analogously, digitally, now in title of SJI
          if debug then stringtime = strtime(time+debugtime) else stringtime = strtime(time)
          if debug then stringtimesec = strtimesec(time+debugtime) else stringtimesec = strtimesec(time)
          xyouts, 0.99, 1-1.7*titleposy, 'Time: '+stringtime, color=0, /normal, charsize=titlesize, charthick=bold, alignment=1, width=w1
          xyouts, 0.98-w1, 1-1.7*titleposy, 'Time [s]: '+stringtimesec, color=0, /normal, charsize=titlesize, charthick=bold, alignment=1, width=w2
          xyouts, 0.97-w1-w2, 1-1.7*titleposy, 'Index: '+string(step,format='(I6.4)'), color=0, /normal, charsize=titlesize, charthick=bold, alignment=1
          clock, stringtime, pos=PosClock[0:1], size=PosClock[2:3], col=255
          
          
          
          
          ; save image
          if saveplot then begin
            file=datestamp + '/' + fns('AApic_####',step) + '.png'
            write_png,file,tvrd(),r,g,b
          endif
          
          
          
          
          
          print, 'step: ', strcompress(string(step), /remove_all), $
            '  time: ', strcompress(string(time), /remove_all), $
            ;'  localtimefuv: ', strcompress(string(localtimefuv), /remove_all), $
            ;'  localtimenuv: ', strcompress(string(localtimenuv), /remove_all), $
            '  PZT: ', strcompress(string(PZTcurrent[0]), /remove_all), $
            ' / ', strcompress(string(PZTcurrent[1]), /remove_all)
            
          step = step + 1
          if debug && (step eq nrpics) then break
        endfor ;nrFDB=0,FRMList[useFRM].NumEntries-1
        
        ;change the position of the origin
        PZT = PZT + PZTStep
        
        ;add a FRM cadence to time
        timeFRMrep = timeFRMrep + FRMcadence
        if timeFRMrep gt time then begin
          localtimefuv = localtimefuv + timeFRMrep - time
          localtimenuv = localtimenuv + timeFRMrep - time
          time = timeFRMrep
        endif
        
        if debug && (step eq nrpics) then break
      endfor ;nrFRMrepeats=0,(*(OBSList).Repeat_FRM)[nrFRM]-1
      
      if debug && (step eq nrpics) then break
    endfor ;nrFRM=0,OBSList.NumEntries-1
    
    if debug && (step eq nrpics) then break
  endfor ;nrOBSrepeats=0,OBSList.Repeat_Obs-1
  
  print, 'End Time: ', time
  
  set_plot,'x'
  !P.Multi=0
  
  if savelogcsv || savelogtxt then begin
    writeLogFile, savelogcsv, savelogtxt, logFilecsvName, logFiletxtName, csvUnit, txtUnit, 0, 1, time
  endif
  
  if makemovie then begin
    file=fns('AApic_####.png',0)
    cd,datestamp
    framerate='--sequencerate=30'
    print,'/usr/local/bin/qt_export --replacefile --video=png,30,100 '+framerate+' '+file+' AAmovie.mov'
    spawn,'/usr/local/bin/qt_export --replacefile --video=png,30,100 '+framerate+' '+file+' AAmovie.mov'
    if playmovie then begin
      print,'opening movie...'
      ;spawn,'/Applications/Utilities/Quicktime\ Player\ 7.app/Contents/MacOS/Quicktime\ Player\ 7 *.mov &'
      spawn, quicktimeloc + ' Player *.mov &'
    endif
    cd,'..'
  endif
end
